//->LightCtrl.arc<@ignoreComments
package ila;
import ila.signals.*;

/**
* Component LightCtrl controls the interior light of a car.
* It receives the status from the light switch via port
* {@link switchStatus}, the status from the alarm system via port
* {@link alarmStatus}, and the car's door state via port
* {@link doorStatus}. Based on this input it calculates and emits
* a command signal via port {@link cmd} to turn the light on or off.
* 
* If the light is switched to door dependent and:
* <ul>
* <li>at least one door is opened, the light is turned on</li>
* <li>the last door is closed, the light is turned off after a short delay of <b>fadeOutTime</b> time units</li>.
* </ul>
* If the alarm system is active, the interior lights blink (alternating ON/OFF 
* each <b>fadeOutTime + 2</b> time units).
*
* @param fadeOutTime fade out time, if door is closed
*/
component LightCtrl[int fadeOutTime]  {
  autoinstantiate on;
  autoconnect port;
  
  port 
    /** Current status of the interior light switch. */
    in SwitchStatus,
    /** Current alarm status of the car. */
    in AlarmStatus,
    /** Current state of the car's doors: at least one open or all closed.*/
    in DoorStatus,
    /** Computed command to control the interior light. */
    out OnOffCmd cmd;
    
 /**
  * Arbitrates between the received inputs with severity:
  * {@link blinkRequest} > {@link onOffRequest} > {@link switchStatus}.
  * The most severe signal is used to compute and emit an OnOffCmd
  * the is emited via port {@link onOffCmd}.
  */
  component Arbiter {
//->LightCtrl.arc<@ignoreComments  
    timing causalsync;
//->LightCtrl.arc<@ignoreComments      
    port
      /** Current status of the interior light switch. */
      in SwitchStatus,
      /** Computed blink request. */
      in BlinkRequest,
      /** Computed on or off request. */
      in OnOffRequest,
      /** Calculated on or off command. */ 
      out OnOffCmd; 
  }

  component AlarmCheck(fadeOutTime + 2) ac;
  component DoorEval(fadeOutTime);
  
  connect arbiter.onOffCmd -> cmd;
}
//->LightCtrl.arc<